/*
 * This is the implementations of all the methods of the system and monitor
 */

#include "./ctl_system_monitor_task.h"
#include "../msp_blimp/_lib_system.h"
#include "../uart/_msp_uart_controller.h"
#include "../ctl/ctl_motor_control_task.h"
#include "./ctl_sensing_task.h"
#include <stdint.h>
#include  "../pid/pid_ctl.h"

#define CLEAR_VAL_FRONT 17800
#define CLEAR_VAL_LEFT_SIDE 17100
#define CLEAR_VAL_RIGHT_SIDE 15000
#define TURN_SPEED 150
#define ENCOUNTER 17000
#define TURNING_COUNTER 200

void handleFacingObject (void);
void handleNotFacingObject (void);
// helper functions used to enable and disable pid based on demmand
void ctl_sys_disablePid (void);
void ctl_sys_enablePid (void);
// helper functions used to enable and disable collision avoidance
void ctl_sys_enableCa (void);
void ctl_sys_disableCa (void);

static void updateMotorState (MOTOR_STAT *mot, uint8_t dir, uint8_t dc ){
	mot->direction = dir;
	mot->duty_cyle = dc;
}

void ctl_sys_forceReset (void){
	// call Denny code to reset the whole system (by writing to WDT)
	// CODE:

	// change the state of the
	ctl_System_State = STOP;
	ctl_Connection_State = DISCONNECTED;
}

void ctl_sys_Run (void){
	if (ctl_System_State != RUNNING) {
		// switch motors and sensors from safe-power mode to running mode
		// change the state of the blimp to run
		ctl_System_State = RUNNING;
	}
}

void ctl_sys_Stop (void){
	// switch sensors and motors to safe-power mode
	// CODE:
	// switch to STOP state
	ctl_System_State = STOP;
	ctl_sys_disableCa();
	ctl_sys_disablePid();
	// disable the pid
	facing_object = 0;
	turning_dir = 0;
	pid_counter = 0;
	collision_counter = 0;
	turn_counter = 0;
	// disable all the motors
	sys_LMotor_forward (0);
	sys_LMotor_reverse (0);

	sys_RMotor_forward (0);
	sys_RMotor_reverse (0);

	sys_VMotor_forward (0);
	sys_VMotor_reverse (0);

	// update the motor speedd to 0
	updateMotorState (&ctl_motor_left, 0, 0);
	updateMotorState (&ctl_motor_right, 0, 0);
	updateMotorState (&ctl_motor_vertical, 0, 0);
}

uint8_t ctl_sys_checkConnectionStatus (void){
	// not connected
	if (ctl_Connection_State != CONNECTED){
		// put to safe mode
		ctl_sys_toFailSafeMode ();
	}
	return CONNECTED;
}

void ctl_sys_resetWDTcounter (void){
	// reset WDT to make it happy
	// CODE:

}

void ctl_sys_toFailSafeMode (void){
	// put the system to fail safe mode
	// CODE:

}


void ctl_sys_updatePidTarget (void){
	// change the target pid value
	unsigned int val = ((unsigned int)(cmd.arg[0])) << 8;
	pid_update_target(&ctl_pid_controller, val);
}
void ctl_sys_updatePid (void){
	// poll the DOWN_SENSOR
	volatile uint32_t val = ctl_sens_PollIR(DOWN_SENSOR);

	//	// send back Sensor value
	//	char s [13] = {'\0'};
	//	s[11] = '\r';
	//	s[12] = '\n';
	//	itoa(val, s);
	//	uart_Transmit_data(s, 13);

	// update the pid controller
	pid_update(&ctl_pid_controller, val);
	// get the speed of the motor
	int duty_cycle = pid_get_output(&ctl_pid_controller);
	// check the duty cycle value to set the appropriate direction and duty cycle
	if (duty_cycle < 0){
		uint8_t real_duty_cycle = duty_cycle * (-1);
		ctl_moto_PidSetVMotor(real_duty_cycle, CMD_MOT_DIR_DOWN);
	}
	else {
		uint8_t real_duty_cycle = duty_cycle;
		ctl_moto_PidSetVMotor(real_duty_cycle, CMD_MOT_DIR_UP);
	}
}

void ctl_sys_pidControl (void){
	// disable pid
	if (cmd.arg[0] == CMD_PID_OFF)
		ctl_sys_disablePid();
	// enable pid
	else if (cmd.arg[0] == CMD_PID_ON)
		ctl_sys_enablePid();
	else
		return ; // this should not happen
}

void ctl_sys_collisionControl (void){
	// disable pid
	if (cmd.arg[0] == 0)
		ctl_sys_disableCa();
	// enable pid
	else if (cmd.arg[0] == 1 )
		ctl_sys_enableCa();
	else
		return ; // this should not happen
}


void ctl_sys_enableCa(void){
	collision_enable = 1;
	facing_object = NOT_FACING_OBJECT;
	turning_dir = NOT_TURNING;
	ctl_System_State = AUTO;
	collision_counter = 0;
	// go forward
	ctl_moto_GoForward();
}

void ctl_sys_disableCa(void){
	collision_enable = 0;
	ctl_sys_Stop();
	//	ctl_moto_SetRightMotor(0, CMD_MOT_DIR_FWD);
	//	ctl_moto_SetLeftMotor(0, CMD_MOT_DIR_FWD);
	//	ctl_System_State = STOP;
	// stop the side motors

}

void ctl_sys_disablePid (void){
	// disable pid global varialble
	pid_enable = CMD_PID_OFF;
	// turn off the vertical motor also
	ctl_moto_PidSetVMotor(0, CMD_MOT_DIR_UP);
}

void ctl_sys_enablePid (void){
	pid_enable = CMD_PID_ON;
	// reset the counter to 0
	pid_counter = 0;
}

void ctl_sys_updateCollision (void){
	// poll the DOWN_SENSOR
	//volatile uint32_t val_forward = ctl_sens_PollIR(FORWARD_SENSOR);
	//volatile uint32_t val_left = ctl_sens_PollIR(LEFT_SENSOR);
	volatile uint32_t val_right = ctl_sens_PollIR(RIGHT_SENSOR);
	//volatile uint32_t val_left = 0;
	//volatile uint32_t val_right = 0;


	//	// send back Sensor value
	char s [13] = {'\0'};
	s[11] = '\r';
	s[12] = '\n';
	//itoa(val_forward, s);
	//itoa(val_left, s);
	itoa(val_right, s);
	uart_Transmit_data(s, 13);
	// check which state we are in
	switch (facing_object){
	case FACING_OBJECT: handleFacingObject ();
	case NOT_FACING_OBJECT: handleNotFacingObject ();
	}
}

void handleNotFacingObject (void){
	// check if we are facing any oject
	volatile uint32_t val_forward = ctl_sens_PollIR(FORWARD_SENSOR);
	if (val_forward <= ENCOUNTER)
		facing_object = FACING_OBJECT;
}

void handleFacingObject (void){
	volatile uint32_t val_forward = ctl_sens_PollIR(FORWARD_SENSOR);
	volatile uint32_t val_left = ctl_sens_PollIR(LEFT_SENSOR);
	volatile uint32_t val_right = ctl_sens_PollIR(RIGHT_SENSOR);
	// check if we got away from the object
	if (turning_dir == TURNING_LEFT){
		// turn off motor if we did not see object any more
		//if (val_right >= CLEAR_VAL_RIGHT_SIDE){
		turn_counter++;
		if (turn_counter == TURNING_COUNTER){
			turn_counter = 0;
			turning_dir  = NOT_TURNING;
			facing_object = NOT_FACING_OBJECT;
			// turn off motor go forward
			ctl_moto_SetLeftMotor(0,CMD_MOT_DIR_REV);
			ctl_moto_SetRightMotor(0,CMD_MOT_DIR_REV);
			ctl_moto_GoForward();
		}

	}else if (turning_dir == TURNING_RIGHT ){
		//if (val_left >= CLEAR_VAL_LEFT_SIDE){
		turn_counter++;
		if (turn_counter == TURNING_COUNTER){
			turn_counter = 0;
			turning_dir  = NOT_TURNING;
			facing_object = NOT_FACING_OBJECT;
			// just turn off the motor if we get away from the object
			ctl_moto_SetLeftMotor(0,CMD_MOT_DIR_REV);
			ctl_moto_SetRightMotor(0,CMD_MOT_DIR_REV);
			// TODO make forward better and recognizable
			ctl_moto_GoForward();
		}
	} else {// if not turning yet need to pick a direction{
		// back up until we clear and start turning
		ctl_moto_GoBackward();
		if (val_forward >= CLEAR_VAL_FRONT){
			// turn right
			if (val_left <= val_right){
				turning_dir  = TURNING_RIGHT;
				ctl_moto_SetRightMotor(TURN_SPEED,CMD_MOT_DIR_FWD);
				ctl_moto_SetLeftMotor(TURN_SPEED,CMD_MOT_DIR_REV);

			}else {// turn left
				turning_dir  = TURNING_LEFT;
				ctl_moto_SetLeftMotor(TURN_SPEED,CMD_MOT_DIR_FWD);
				ctl_moto_SetRightMotor(TURN_SPEED,CMD_MOT_DIR_REV);
			}
		}
	}

}





