
/*
 * This module is used to handle the System and Monitor Task of the blimp bot such as:
 * Checking Run, stop
 * Checking Connected or Disconnected
 * Force Reset
 * Reset WDT counter to make it happy !!
 */


#ifndef _CTL_SYSTEM_MONITOR_TASK_
#define _CTL_SYSTEM_MONITOR_TASK_

#include "./global.h"
#include "../utils/itoa.h"



// command struct contains the required args
extern int turn_counter;
extern CMD_COMMAND_FRAME cmd;

extern uint8_t facing_object;
extern uint8_t turning_dir;
extern int collision_counter;
extern uint8_t collision_enable;
// represent system State
extern uint8_t ctl_System_State;
// represent system connection
extern uint8_t ctl_Connection_State;
// this function is used to enforce the reset

extern MOTOR_STAT ctl_motor_left;
extern MOTOR_STAT ctl_motor_right;
extern MOTOR_STAT ctl_motor_vertical;

extern PID_INFO ctl_pid_controller;
// attemping to write to WDT register
void ctl_sys_forceReset (void);

// check for connection by sending a ping / keepalive message
// if not CONNECTED => put the motor and system to safemode by
uint8_t ctl_sys_checkConnectionStatus (void); 

// reset WDT counter at the beginning of every rung
void ctl_sys_resetWDTcounter (void);
 
// enable sensor and motor for running
// update isRunning status
void ctl_sys_Run (void);

// set to safe-power down state all the sensor and motor
// update isRunning status
void ctl_sys_Stop (void);

// to fail-safe mode when communication is lost
void ctl_sys_toFailSafeMode (void);

// update pid periodically
void ctl_sys_updatePid (void);

//  update pid target
void ctl_sys_updatePidTarget (void);

// update facing object
void ctl_sys_updateCollision (void);


// disable pid when need to
void ctl_sys_pidControl (void);

void ctl_sys_collisionControl (void);

// to return system status to the application side
//void ctl_sys_query

#endif // _CTL_SYSTEM_MONITOR_TASK_
