/* this is the main control of the Blimp */

#include "./wci/wifi_ctrl_iface.h"
#include <stdint.h>
#include "./ctl/ctl_system_monitor_task.h"
#include "./ctl/ctl_motor_control_task.h"
#include "./ctl/ctl_communication_task.h"
#include "./msp_blimp/_lib_system.h"
#include "./uart/_msp_uart_controller.h"
#include "./pid/pid_ctl.h"
#include "./ctl/ctl_sensing_task.h"


// this function is used to initialize the system
static void ctl_Initialization (void);
// method used for decoding and executing command
static void ctl_CommandExecution (CMD_COMMAND_FRAME *cmd);
/*----------SYSTEM STATEs------------------------------------------*/
// represent system State
uint8_t ctl_System_State = STOP;
// represent system connection
uint8_t ctl_Connection_State = CONNECTED;

/*-----------------MOTOR CONTROL STATEs-----------------------------------*/
MOTOR_STAT ctl_motor_left = {MOTOR_LEFT, 0, 0};
MOTOR_STAT ctl_motor_right = {MOTOR_RIGHT, 0, 0};
MOTOR_STAT ctl_motor_vertical=  {MOTOR_VERT, 0, 0};

/*----------------PID CONTROLLER ------------------------------------------*/

#define PID_INIT_P 1
#define PID_INIT_I 0.1
#define PID_INIT_D 0.5
unsigned int PID_INIT_TARGET = 15000;
PID_INFO ctl_pid_controller;

/*--------timer counter used to timeout when connection is lost -----------*/
#define TIME_OUT 50000
#define PID_COUNTER 1000
int timer_counter = 0;
int pid_counter = 0;
uint8_t pid_enable = 0;
/*---------HELPER FUNCTIONS -------------*/
//static void itoa(unsigned int input, char *a);
//static void Reverse  (char* s, int length);
/*--------------------MAIN PROGRAM -----------------------------------------*/
int main (void){
	// do some initialization
	ctl_Initialization();
	// enter the loop for execution command
	//while (timer_counter <= TIME_OUT ){
	while (1){
		// TODO : figure out how to checking connection status using timer
		// TURN OFF RED LED FOR TESTING PURPOSE
		// read command from UART
		pid_counter++;
		if (wci_hasNext()) {
			// reset timer counter whenever receive new task
			timer_counter = 0;
			GREEN_LED_TOGGLE;
			// get the command frame from the uart
			CMD_COMMAND_FRAME cmd;
			wci_nextCommand (&cmd);
			// if not running look for run command

			//if ((ctl_System_State == STOP) && (cmd.OPcode != CMD_OPCODE_RUN))
			//	continue;

			// otherwise execute command
			ctl_CommandExecution(&cmd);
		}else{ // if there is no communication from the uart increase the counter
			timer_counter++;
		}
		/*TODO: POLLING IR sensor for PID controller */
		if ((ctl_System_State == RUNNING) && (pid_enable) && (pid_counter > PID_COUNTER)){
			// reset the pid timer
			pid_counter = 0;
			// update pid
			ctl_sys_updatePid();
		}
	}
}

void ctl_CommandExecution (CMD_COMMAND_FRAME *cmd){
	// command only execute at running state
	if (ctl_System_State == RUNNING){
		switch (cmd->OPcode){
		case (CMD_OPCODE_MOTOR): ctl_moto_SetMotor(cmd); return;
		case (CMD_OPCODE_STOP): ctl_sys_Stop(); return;
		default: return;
		}
	}
	// always execute command
	switch (cmd->OPcode){
	case (CMD_OPCODE_RUN): ctl_sys_Run(); return;
	case (CMD_OPCODE_PING): ctl_comm_ping_response (); return; // Ping now can response back
	case (CMD_OPCODE_POLL): ctl_sens_PollIR (DOWN_SENSOR); return;
	default: return;
	}
}

void ctl_Initialization (void){
	// call all initialize functions
	// CODE:
	sys_Initialize();
	// initialize uart
	uart_Initialize();
	uart_Enable_RX_interrupt();
	// initialize IR sensor
	sensor_Initialize();
	// initialize the pid
	pid_init(&ctl_pid_controller, PID_INIT_P, PID_INIT_I, PID_INIT_D, PID_INIT_TARGET);
}










