/* this is the main control of the Blimp */

#include "./wci/wifi_ctrl_iface.h"
#include <stdint.h>
#include "./ctl/ctl_system_monitor_task.h"
#include "./ctl/ctl_motor_control_task.h"
#include "./ctl/ctl_communication_task.h"
#include "./msp_blimp/_lib_system.h"
#include "./uart/_msp_uart_controller.h"
#include "./pid/pid_ctl.h"
#include "./ctl/ctl_sensing_task.h"


// this function is used to initialize the system
static void ctl_Initialization (void);
/*----------SYSTEM STATEs------------------------------------------*/
#define TIME_OUT 50000
// represent system State
uint8_t ctl_System_State = STOP;
// represent system connection
uint8_t ctl_Connection_State = CONNECTED;
uint8_t facing_object = 0;
uint8_t turning_dir = 0;
/*-----------------MOTOR CONTROL STATEs-----------------------------------*/
MOTOR_STAT ctl_motor_left = {MOTOR_LEFT, 0, 0};
MOTOR_STAT ctl_motor_right = {MOTOR_RIGHT, 0, 0};
MOTOR_STAT ctl_motor_vertical=  {MOTOR_VERT, 0, 0};

/*----------------PID CONTROLLER ------------------------------------------*/

#define PID_INIT_P 20 //2
#define PID_INIT_I 0 // 0
#define PID_INIT_D 5
uint32_t PID_INIT_TARGET = 17000;
uint32_t FRONT_INIT_TARGET = 14000;
PID_INFO ctl_pid_controller;
/*--------timer counter used to timeout when connection is lost -----------*/
#define PID_COUNTER 50
#define COLLISION_COUNTER 200

int timer_counter = 0;
// these variables are used for control pid
int pid_counter = 0;
int turn_counter = 0;
int collision_counter = 0;
uint8_t pid_enable = 0;
// enable or disable collision avoidance
uint8_t collision_enable = 0;
/*---------HELPER FUNCTIONS -------------*/
void ctl_NextState (void );
void ctl_ConnectionTask (void);
void ctl_PIDTask (void);
void ctl_CollisionAvoidanceTask (void);
// calling the appropriate function to execute
void ctl_NextState (void );
// check if is there anytask need to be executed
void ctl_CheckScheduleTask (void);
void nothing_func (void);
/*----------------GLOBAL COMMAND STRUCT ----------------*/
CMD_COMMAND_FRAME cmd;
//static void itoa(unsigned int input, char *a);
//static void Reverse  (char* s, int length);
/*--------------------MAIN PROGRAM -----------------------------------------*/
int main (void){
	// do some initialization
	ctl_Initialization();
	// enter the loop for execution command
	//while (timer_counter <= TIME_OUT ){
	while (1){
		// TODO : figure out how to checking connection status using timer
		// TURN OFF RED LED FOR TESTING PURPOSE
		// read command from UART
		if (wci_hasNext()) {
			// reset timer counter whenever receive new task
			timer_counter = 0;

			// toggle LED when received some command
			GREEN_LED_TOGGLE;
			// get the command frame from the uart
			wci_nextCommand (&cmd);
			// change to next state based on event driven
			ctl_NextState ();
		}else{ // if there is no communication from the uart increase the counter
			timer_counter++;
		}
		/* Check if is there any task need to be execute */
		ctl_CheckScheduleTask ();
	}
}

/*  ---------------------------------STATE MACHINE && HELPER FUNCTIONS-------------------------------------------------------*/
typedef void (*BLIMP_FUNCTION) (void);

// this fake function used to fill in the state function table
void nothing_func (void){}

BLIMP_FUNCTION StateFunc [3][14] ={

		// STOP STATE
		//NOP				// RUN,			    STOP,			RESET,				PING_FUNC,
		{&nothing_func,  &ctl_sys_Run,  &nothing_func,    &ctl_sys_Stop,	&ctl_comm_ping_response,
		//STAT				//MOTOR		// SET			// TUNE		// POLL
		&nothing_func, 	&nothing_func, &nothing_func, &nothing_func,  &nothing_func,
		// QUERRY 		// PID					// CA						// TAR
		&nothing_func,   &ctl_sys_pidControl, 	&ctl_sys_collisionControl, 	&ctl_sys_updatePidTarget},

		// RUNNING STATE
		//NOP				// RUN,			    STOP,			RESET,  			PING
		{&nothing_func,  &ctl_sys_Run,	&ctl_sys_Stop,	&ctl_sys_Stop, 	&ctl_comm_ping_response,
			//STAT				//MOTOR		// SET			// TUNE		// POLL
		&nothing_func,	&ctl_moto_SetMotor, &nothing_func, &nothing_func,  &nothing_func,
		//QUERRY		   // PID				 	 // CA					//TAR
		&nothing_func,    &ctl_sys_pidControl, 	&ctl_sys_collisionControl, &ctl_sys_updatePidTarget},

		// AUTO STATE
		//NOP				// RUN,			    STOP,			RESET,  			PING
		{&nothing_func,  &nothing_func,	&ctl_sys_Stop,	&ctl_sys_Stop, 	&ctl_comm_ping_response,
		//STAT				//MOTOR		// SET			// TUNE		// POLL
		&nothing_func,	&nothing_func, &nothing_func, &nothing_func,  &nothing_func,
		//QUERRY		   // PID				 // CA					    // TAR
		&nothing_func,    &ctl_sys_pidControl, 	&ctl_sys_collisionControl, &ctl_sys_updatePidTarget}




};


void ctl_NextState (void ){
	// execute the corresponding function for the current state
	(StateFunc[ctl_System_State][cmd.OPcode]());
}

/*----------------------------------------FUNCTION TASK -------------------------------------------------------------*/

void ctl_CheckScheduleTask (void){
	if (ctl_System_State == RUNNING || ctl_System_State == AUTO){
		// checking PID task
		ctl_PIDTask();
		// TODO: checking Connection stat to set the blimp to safe state if lost connection
		//ctl_ConnectionTask();
		ctl_CollisionAvoidanceTask();
	}
}

void ctl_PIDTask (void){
	// check global condition in order to update PID if neccessary
	if (pid_enable){
		pid_counter++;
		if (pid_counter >= PID_COUNTER){
			pid_counter = 0;
			ctl_sys_updatePid();
		}
	}
}

void ctl_ConnectionTask (void){
	// check the timer
	if (timer_counter == TIME_OUT){
	// stop everything and assume connection is lost
		ctl_sys_Stop();
	}
}

void ctl_CollisionAvoidanceTask (void){
	if (collision_enable){
		collision_counter++;
		if (collision_counter >= COLLISION_COUNTER){
			collision_counter = 0;
			ctl_sys_updateCollision();
		}
	}


}
/*----------------------------------------------------------------------------------------------------------------------*/


void ctl_Initialization (void){
	// call all initialize functions
	// CODE:
	sys_Initialize();
	// initialize uart
	uart_Initialize();
	uart_Enable_RX_interrupt();
	// initialize IR sensor
	sensor_Initialize();
	// initialize the pid
	pid_init(&ctl_pid_controller, PID_INIT_P, PID_INIT_I, PID_INIT_D, PID_INIT_TARGET);
	// set the blimp to go forward if collision avoidance is set to enable
	//if (collision_enable)
		//ctl_moto_GoForward();
}












