// System-level library, such as GPIOs initialization or watchdog controll
// 
// Author: Antonius Denny Harijanto
//

#ifndef _lib_system
#define _lib_system

#include <msp430f5510.h>
#include <stdint.h>

// ----------------------------------------------------------------------
// LED CONTROL
// ----------------------------------------------------------------------
// LED6
#define RED_LED_SETUP P5DIR |=0X2 ; P5OUT &= ~0x2
#define RED_LED_TOGGLE P5OUT ^= 0X2
#define RED_LED_ON P5OUT |= 0X2
#define RED_LED_OFF P5OUT &= ~0X2

// LED 7
#define GREEN_LED_SETUP PJDIR |=0x8 ; PJOUT &= 0x8
#define GREEN_LED_TOGGLE PJOUT ^= 0x8
#define GREEN_LED_ON PJOUT |= 0x8
#define GREEN_LED_OFF PJOUT &= ~0x8
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// SYSTEM INITIALIZATION
// ----------------------------------------------------------------------

// Initializes the pins and macro for
// MSP430f5510 on the Blimp side.
// This function does these:
//  + Disable watchdog timer
//  + Setup CPU speed = 12MHz
//  + Setup PINs for LEDs
//
// This is the first function that needs to be called
// before any other things. 
// This would be the first function to be 
// called when Blimp program starts
void sys_Initialize();
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// WATCHDOG CONTROL
// TODO : haven't been implemented
// ----------------------------------------------------------------------
// Disable/Enable watchdog timer
void sys_Disable_watchdog();
// period is the timemout of the watchdog
void sys_Enable_watchdog(uint16_t period);
// Reset watchdog timer
// If the watchdog is activated, this would 
// need to be called periodically, otherwise
// system reset would be forced
void sys_Reset_watchdog();
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// MOTOR CONTROL
// ----------------------------------------------------------------------
// CCR0 value for TIMERA, which is the PWM period
// Make this bigger to slow down the motor frequency
#define MOTOR_PWM_PERIOD_SHIFT 11
#define MOTOR_PWM_PERIOD (1 << MOTOR_PWM_PERIOD_SHIFT)

// Run the motor with the specified duty cycle
// power is ratio of the motor frequency (0-255)
// 0 is off, while 255 is full power
void sys_VMotor_forward(uint32_t freq);
void sys_VMotor_reverse(uint32_t freq);

void sys_LMotor_forward(uint32_t freq);
void sys_LMotor_reverse(uint32_t freq);

void sys_RMotor_forward(uint32_t freq);
void sys_RMotor_reverse(uint32_t freq);

#endif