#include "../ctl/global.h"
#include "./pid_ctl.h"




void pid_init(struct pid *controller, float p, float i, float d, unsigned int target)
{
  controller->Kp = p;
  controller->Ki = i;
  controller->Kd = d;
  controller->target_value = target;
  controller->sensor_old = controller->sensor_new = 0;
  controller->error = controller->delta_sensor = controller->integral_sum = 0;
}

void pid_reset(struct pid *controller)
{
  controller->sensor_old = controller->sensor_new = 0;
  controller->error = controller->delta_sensor = controller->integral_sum = 0;
}

void pid_update_target(struct pid *controller, unsigned int target)
{
  pid_reset(controller);
  controller->target_value = target;

}

void pid_update_Kp(struct pid *controller, float c)
{
  controller->Kp = c;
}

void pid_update_Ki(struct pid *controller, float c)
{
  controller->Ki = c;
}

void pid_update_Kd(struct pid *controller, float c)
{
  controller->Kd = c;
}

void pid_update(struct pid *controller, unsigned int reading)
{
  controller->sensor_old = controller->sensor_new;
  controller->sensor_new = reading;
  controller->delta_sensor = reading - controller->sensor_old;
  controller->integral_sum += reading;
  controller->error = controller->target_value - reading;
}

int pid_get_output(struct pid *controller) 
{
  float Vp = controller->Kp * controller->error;
  float Vi = controller->Ki * controller->integral_sum;
  float Vd = controller->Kd * controller->delta_sensor;
  int output = (int)(Vp + Vi + Vd);
  if (output > MOTOR_MAX)
    return MOTOR_MAX;
  else if (output < MOTOR_MIN)
    return MOTOR_MIN;
  return output;
}
