// This header contains the definitons of the UART that the Blimp
// uses for communication with the ez430RF2500 micro-controller attached to it (the wifi board)
// 
// Author: Antonius Denny Harijanto
// 

#ifndef _MSP_UART_CONTROLLER_
#define _MSP_UART_CONTROLLER_

#include <stdint.h>
#include <msp430f5510.h>
#include "../msp_blimp/_lib_system.h"
#include "../utils/queue.h"


//-----------------------------------------------------------------------------------------
// Functions
//-----------------------------------------------------------------------------------------
// Initializes the UART functionality on the Blimp
// This function assumes the following:
//  - The port used for UART is P4.4 and P4.5
//  - CPU clock (SMCLK) is already set to 12MHz
//  - 
// This function mutates the microcontroller as follow:
//  + Enable UCA1 read and transmitt interrupt
//  + Enable the global interrupt
// 
// Parameters:
// dataReceiverFn callback function when the data is received
//                data is the data received while length is the data length
//
void uart_Initialize(void);

// Enable/disable UART read interrupt
void uart_Disable_RX_interrupt(void);
void uart_Enable_RX_interrupt(void);

// Enable/disable UART write interrupt
void uart_Disable_TX_interrupt(void);
void uart_Enable_TX_interrupt(void);

// Send data over the UART by placing the data in the
// transmit queue. This queue would be processed by TX ISR
//
// Params:
// -------
// str the data to be sent over the UART
// length how big the data to be sent is
//
// Return:
// -------
// 1 if the data were able to be put on the queue, or 0 otherwise
uint8_t uart_Transmit_data(char *str, uint8_t length);

#endif
