// Matthew Kraus
// fsm.h -- finite state machine for embedded systems
// module prefix: fsm_
//

#pragma once

#ifndef INCLUDED_FSM
#define INCLUDED_FSM

#ifndef INCLUDED_MSP430_DATATYPES
	#include "msp430_datatypes.h"
#endif

// things that can happen in any given state
// define all of these to fully specify the state
typedef struct FSM_STATE {
	BOOL (*acceptsInput)(void* rawData);      // true if this state accepts this input
	struct FSM_STATE** nextState;             // an array of pointers to possible next states
	uint8_t numNextStates;
   
	struct FSM_STATE* defaultState;           // switches to this state if there are no other accepting states for the current input
	                                          // ...cannot be null
   
	void (*onEnter)(void* rawData);           // executed when entering this state (not executed if null); can be null
	void (*onExit)(void* rawData);            // executed when leaving this state (not executed if null); can be null
} FSM_STATE;

void fsm_init(FSM_STATE* currentState, FSM_STATE* startingState);

void fsm_onInput(FSM_STATE* currentState, void* rawInput);

#endif // INCLUDED_FSM